import { __assign, __decorate, __param } from 'tslib';
import { isPlatformBrowser } from '@angular/common';
import { InjectionToken, Inject, Injectable, Renderer2, PLATFORM_ID, Optional, Input, Component, ChangeDetectionStrategy, HostBinding, EventEmitter, ElementRef, Output, Directive, NgModule } from '@angular/core';
import { WINDOW } from 'ngx-window-token';
import { DomSanitizer } from '@angular/platform-browser';

var IMAGE_LOADER_CONFIG_TOKEN = new InjectionToken('Image loader Config');
var DEFAULT_IMAGE_LOADER_OPTIONS = {
    // root?: Element | null;
    rootMargin: '10px',
    threshold: [0.1, 0.5, 1],
    imageRatio: 16 / 9,
    placeholderImageSrc: '',
    isAggressiveLoading: true,
    concurrentLoading: 4
};

var ConfigurationService = /** @class */ (function () {
    function ConfigurationService(options) {
        this.setConfig(options);
    }
    ConfigurationService.prototype.setConfig = function (config) {
        if (config) {
            this.config = __assign(__assign({}, DEFAULT_IMAGE_LOADER_OPTIONS), config);
        }
        else {
            this.config = DEFAULT_IMAGE_LOADER_OPTIONS;
        }
    };
    ConfigurationService.ctorParameters = function () { return [
        { type: undefined, decorators: [{ type: Inject, args: [IMAGE_LOADER_CONFIG_TOKEN,] }] }
    ]; };
    ConfigurationService = __decorate([
        Injectable(),
        __param(0, Inject(IMAGE_LOADER_CONFIG_TOKEN))
    ], ConfigurationService);
    return ConfigurationService;
}());

function isSupportIntersectionObserver(window) {
    return (window &&
        'IntersectionObserver' in window &&
        'IntersectionObserverEntry' in window &&
        'intersectionRatio' in window.IntersectionObserverEntry.prototype);
}
function isImagePicture(element) {
    return element instanceof HTMLImageElement || element instanceof HTMLPictureElement;
}
function setAttribute(renderer, attribute, element) {
    renderer.setAttribute(element, attribute, element.dataset[attribute]);
    // maybe doesn't matter
    // renderer.removeAttribute(element, 'data-' + attribute);
}
function isPictureElement(element) {
    return element.nodeName === 'PICTURE';
}
function loadImage(renderer, image) {
    if (isPictureElement(image.parentElement)) {
        var sourceElms = image.parentElement.children;
        for (var index = 0; index < sourceElms.length; index++) {
            var element = sourceElms[index];
            if (element instanceof HTMLSourceElement) {
                setAttribute(renderer, 'srcset', element);
            }
            else if (element instanceof HTMLImageElement) {
                setAttribute(renderer, 'src', element);
            }
        }
    }
    else {
        if (image.dataset.src) {
            setAttribute(renderer, 'src', image);
        }
        if (image.dataset.srcset) {
            setAttribute(renderer, 'srcset', image);
        }
    }
}
function isSpider(window) {
    return ((window && !('onscroll' in window)) ||
        /(gle|ing|ro)bot|crawl|spider/i.test(window.navigator.userAgent));
}

var ProgressiveImageLoaderComponent = /** @class */ (function () {
    function ProgressiveImageLoaderComponent(_Renderer, _ConfigurationService, platformId, window) {
        this._Renderer = _Renderer;
        this._ConfigurationService = _ConfigurationService;
        this.platformId = platformId;
        this.window = window;
        // to store observed images
        this.targetMap = new Map();
        // to maintain the sequence of observed images
        this.targetQueue = [];
        // counter of current loading images
        this.loading = 0;
    }
    Object.defineProperty(ProgressiveImageLoaderComponent.prototype, "isObservable", {
        get: function () {
            return !!this.intersectionObserver;
        },
        enumerable: true,
        configurable: true
    });
    ProgressiveImageLoaderComponent.prototype.ngOnInit = function () {
        if (this.isAggressiveLoading === undefined) {
            this.isAggressiveLoading = this._ConfigurationService.config.isAggressiveLoading;
        }
        if (this.concurrentLoading === undefined) {
            this.concurrentLoading = this._ConfigurationService.config.concurrentLoading;
        }
        if (this.window &&
            isSupportIntersectionObserver(this.window) &&
            !isSpider(this.window) &&
            isPlatformBrowser(this.platformId)) {
            if (!this.imageRatio) {
                this.imageRatio = this._ConfigurationService.config.imageRatio;
            }
            if (!this.filter) {
                this.filter = this._ConfigurationService.config.filter;
            }
            if (!this.placeholderImageSrc) {
                this.placeholderImageSrc = this._ConfigurationService.config.placeholderImageSrc;
            }
            this.intersectionObserver = new IntersectionObserver(this.onIntersectionChanged.bind(this), this._ConfigurationService.config);
        }
    };
    ProgressiveImageLoaderComponent.prototype.observe = function (target) {
        // so intersection observer can always detect it correctly, otherwise image elements with 0 in height sometime don't load correctly
        target.style.minHeight = '1rem';
        this.intersectionObserver.observe(target);
        this.targetMap.set(target.dataset.src, target);
        this.targetQueue.push(target.dataset.src);
    };
    ProgressiveImageLoaderComponent.prototype.unobserve = function (target) {
        target.style.minHeight = 'initial';
        this.targetMap.delete(target.dataset.src);
        this.intersectionObserver.unobserve(target);
    };
    // called after an image loaded
    ProgressiveImageLoaderComponent.prototype.imageLoaded = function () {
        this.loading--;
        while (this.isAggressiveLoading &&
            this.targetQueue &&
            this.targetQueue.length &&
            this.loading <= this.concurrentLoading) {
            var next = this.targetQueue.shift();
            this.targetMap.has(next) && this.loadImage(this.targetMap.get(next));
        }
    };
    ProgressiveImageLoaderComponent.prototype.onIntersectionChanged = function (entries, observer) {
        var _this = this;
        entries.forEach(function (entry) {
            if (entry.isIntersecting) {
                _this.loadImage(entry.target);
            }
        });
    };
    // start loading an image
    ProgressiveImageLoaderComponent.prototype.loadImage = function (image) {
        // Stop observing the current target
        this.unobserve(image);
        this.loading++;
        loadImage(this._Renderer, image);
    };
    ProgressiveImageLoaderComponent.prototype.reset = function () {
        this.targetQueue = [];
        this.targetMap = new Map();
        this.isObservable && this.intersectionObserver.disconnect();
    };
    ProgressiveImageLoaderComponent.prototype.ngOnDestroy = function () {
        this.isObservable && this.intersectionObserver.disconnect();
        this.intersectionObserver = this.targetQueue = this.targetMap = undefined;
    };
    ProgressiveImageLoaderComponent.ctorParameters = function () { return [
        { type: Renderer2 },
        { type: ConfigurationService },
        { type: undefined, decorators: [{ type: Inject, args: [PLATFORM_ID,] }] },
        { type: undefined, decorators: [{ type: Optional }, { type: Inject, args: [WINDOW,] }] }
    ]; };
    __decorate([
        Input()
    ], ProgressiveImageLoaderComponent.prototype, "imageRatio", void 0);
    __decorate([
        Input()
    ], ProgressiveImageLoaderComponent.prototype, "concurrentLoading", void 0);
    __decorate([
        Input()
    ], ProgressiveImageLoaderComponent.prototype, "isAggressiveLoading", void 0);
    __decorate([
        Input()
    ], ProgressiveImageLoaderComponent.prototype, "filter", void 0);
    __decorate([
        Input()
    ], ProgressiveImageLoaderComponent.prototype, "placeholderImageSrc", void 0);
    ProgressiveImageLoaderComponent = __decorate([
        Component({
            selector: 'ngx-progressive-image-loader',
            exportAs: 'ngxProgressiveImageLoader',
            template: "\n    <ng-content></ng-content>\n  ",
            changeDetection: ChangeDetectionStrategy.OnPush
        }),
        __param(2, Inject(PLATFORM_ID)),
        __param(3, Optional()),
        __param(3, Inject(WINDOW))
    ], ProgressiveImageLoaderComponent);
    return ProgressiveImageLoaderComponent;
}());

var ImagePlaceholderComponent = /** @class */ (function () {
    function ImagePlaceholderComponent(sanitizer, _ProgressiveImageLoader) {
        this.sanitizer = sanitizer;
        this._ProgressiveImageLoader = _ProgressiveImageLoader;
        this.class = 'ngx-image-placeholder';
    }
    Object.defineProperty(ImagePlaceholderComponent.prototype, "placeHolder", {
        get: function () {
            return this.sanitizer.bypassSecurityTrustStyle("padding-bottom: " + 100 / (this.imageRatio || this._ProgressiveImageLoader.imageRatio) + "%;");
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ImagePlaceholderComponent.prototype, "imageFilter", {
        get: function () {
            return this.sanitizer.bypassSecurityTrustStyle("" + this._ProgressiveImageLoader.filter);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ImagePlaceholderComponent.prototype, "safeLoadingImage", {
        get: function () {
            return this.sanitizer.bypassSecurityTrustUrl(this.placeholderImageSrc || this._ProgressiveImageLoader.placeholderImageSrc);
        },
        enumerable: true,
        configurable: true
    });
    ImagePlaceholderComponent.prototype.ngOnInit = function () { };
    ImagePlaceholderComponent.ctorParameters = function () { return [
        { type: DomSanitizer },
        { type: ProgressiveImageLoaderComponent }
    ]; };
    __decorate([
        HostBinding('class')
    ], ImagePlaceholderComponent.prototype, "class", void 0);
    __decorate([
        HostBinding('style')
    ], ImagePlaceholderComponent.prototype, "placeHolder", null);
    __decorate([
        Input()
    ], ImagePlaceholderComponent.prototype, "imageRatio", void 0);
    __decorate([
        Input()
    ], ImagePlaceholderComponent.prototype, "placeholderImageSrc", void 0);
    ImagePlaceholderComponent = __decorate([
        Component({
            selector: 'ngx-image-placeholder',
            template: "\n    <img class=\"placeholder-loading-image\" [src]=\"safeLoadingImage\" [style.filter]=\"imageFilter\" />\n    <ng-content></ng-content>\n  ",
            changeDetection: ChangeDetectionStrategy.OnPush
        })
    ], ImagePlaceholderComponent);
    return ImagePlaceholderComponent;
}());

var ProgressiveImageDirective = /** @class */ (function () {
    function ProgressiveImageDirective(_ElementRef, _Renderer, _ConfigurationService, _ImagePlaceholder, _ProgressiveImageLoader) {
        this._ElementRef = _ElementRef;
        this._Renderer = _Renderer;
        this._ConfigurationService = _ConfigurationService;
        this._ImagePlaceholder = _ImagePlaceholder;
        this._ProgressiveImageLoader = _ProgressiveImageLoader;
        this.noPlaceholder = false;
        this.onImageLoaded = new EventEmitter();
        this.isObserved = false;
    }
    Object.defineProperty(ProgressiveImageDirective.prototype, "imageRatio", {
        get: function () {
            return this._imageRatio || this._ProgressiveImageLoader.imageRatio;
        },
        // to create a placeholder before finish loading the real image to avoid reflow
        set: function (value) {
            this._imageRatio = value;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ProgressiveImageDirective.prototype, "placeholderImageSrc", {
        get: function () {
            return this._placeholderImageSrc || this._ProgressiveImageLoader.placeholderImageSrc;
        },
        set: function (value) {
            this._placeholderImageSrc = value;
        },
        enumerable: true,
        configurable: true
    });
    ProgressiveImageDirective.prototype.ngOnInit = function () {
        var _this = this;
        this.imageElement = this._ElementRef.nativeElement;
        this.setDataSrc('data-src', this.src);
        this.setDataSrc('data-srcset', this.srcset);
        if (this._ProgressiveImageLoader.isObservable) {
            // only image element need to be observe and have onload event
            if (this.imageElement instanceof HTMLImageElement) {
                this.isObserved = true;
                this._ProgressiveImageLoader.observe(this.imageElement);
                this.imageElement.onload = function () {
                    _this.onImageLoaded.emit(_this.imageElement);
                    _this.imageElement.classList.add('loaded');
                    _this._ProgressiveImageLoader.imageLoaded();
                };
                if (!this._ImagePlaceholder && !this.noPlaceholder) {
                    this.setPlaceholder();
                }
            }
        }
        else {
            // show image directly
            loadImage(this._Renderer, this.imageElement);
            this.imageElement.classList.add('loaded');
        }
    };
    ProgressiveImageDirective.prototype.ngOnChanges = function (changes) {
        changes.src && !changes.src.isFirstChange() && this.setDataSrc('data-src', this.src);
        changes.srcset &&
            !changes.srcset.isFirstChange() &&
            this.setDataSrc('data-srcset', this.srcset);
        if (this.isObserved &&
            ((changes.src && !changes.src.isFirstChange()) ||
                (changes.srcset && !changes.srcset.isFirstChange()))) {
            this._ProgressiveImageLoader.unobserve(this.imageElement);
            this._ProgressiveImageLoader.observe(this.imageElement);
        }
    };
    ProgressiveImageDirective.prototype.setDataSrc = function (attr, value) {
        value && this._Renderer.setAttribute(this.imageElement, attr, value);
    };
    ProgressiveImageDirective.prototype.setPlaceholder = function () {
        var parentElement = this.imageElement.parentElement;
        var placeholder = this.createPlaceholder(this.createPlaceholderImage());
        if (isPictureElement(parentElement)) {
            var pictureParent = parentElement.parentElement;
            this.insertPlaceholder(pictureParent, parentElement, placeholder);
        }
        else {
            this.insertPlaceholder(parentElement, this.imageElement, placeholder);
        }
    };
    ProgressiveImageDirective.prototype.insertPlaceholder = function (parentElement, imagePicture, placeholder) {
        parentElement.insertBefore(placeholder, imagePicture);
        placeholder.style.paddingBottom = 100 / this.imageRatio + "%";
        placeholder.appendChild(imagePicture);
    };
    ProgressiveImageDirective.prototype.createPlaceholder = function (placeholderImage) {
        var placeholder = document.createElement('div');
        placeholder.classList.add('ngx-image-placeholder');
        placeholder.appendChild(placeholderImage);
        return placeholder;
    };
    ProgressiveImageDirective.prototype.createPlaceholderImage = function () {
        var img = new Image();
        img.classList.add('placeholder-loading-image');
        img.style.filter = this._ProgressiveImageLoader.filter;
        img.src = this.placeholderImageSrc;
        return img;
    };
    ProgressiveImageDirective.ctorParameters = function () { return [
        { type: ElementRef },
        { type: Renderer2 },
        { type: ConfigurationService },
        { type: ImagePlaceholderComponent, decorators: [{ type: Optional }, { type: Inject, args: [ImagePlaceholderComponent,] }] },
        { type: ProgressiveImageLoaderComponent, decorators: [{ type: Inject, args: [ProgressiveImageLoaderComponent,] }] }
    ]; };
    __decorate([
        Input()
    ], ProgressiveImageDirective.prototype, "imageRatio", null);
    __decorate([
        Input()
    ], ProgressiveImageDirective.prototype, "placeholderImageSrc", null);
    __decorate([
        Input()
    ], ProgressiveImageDirective.prototype, "src", void 0);
    __decorate([
        Input()
    ], ProgressiveImageDirective.prototype, "srcset", void 0);
    __decorate([
        Input()
    ], ProgressiveImageDirective.prototype, "noPlaceholder", void 0);
    __decorate([
        Output()
    ], ProgressiveImageDirective.prototype, "onImageLoaded", void 0);
    ProgressiveImageDirective = __decorate([
        Directive({
            // make sure the element is an image element
            selector: 'img[ngxProgressiveImage], source[ngxProgressiveImage]'
        }),
        __param(3, Optional()),
        __param(3, Inject(ImagePlaceholderComponent)),
        __param(4, Inject(ProgressiveImageLoaderComponent))
    ], ProgressiveImageDirective);
    return ProgressiveImageDirective;
}());

function ConfigurationServiceFactory(options) {
    return new ConfigurationService(options);
}
var NgxProgressiveImageLoaderModule = /** @class */ (function () {
    function NgxProgressiveImageLoaderModule() {
    }
    NgxProgressiveImageLoaderModule_1 = NgxProgressiveImageLoaderModule;
    NgxProgressiveImageLoaderModule.forRoot = function (config) {
        return {
            ngModule: NgxProgressiveImageLoaderModule_1,
            providers: [
                { provide: IMAGE_LOADER_CONFIG_TOKEN, useValue: config },
                {
                    provide: ConfigurationService,
                    useFactory: ConfigurationServiceFactory,
                    deps: [IMAGE_LOADER_CONFIG_TOKEN]
                }
            ]
        };
    };
    var NgxProgressiveImageLoaderModule_1;
    NgxProgressiveImageLoaderModule = NgxProgressiveImageLoaderModule_1 = __decorate([
        NgModule({
            imports: [],
            declarations: [
                ProgressiveImageLoaderComponent,
                ProgressiveImageDirective,
                ImagePlaceholderComponent
            ],
            exports: [ProgressiveImageLoaderComponent, ProgressiveImageDirective, ImagePlaceholderComponent]
        })
    ], NgxProgressiveImageLoaderModule);
    return NgxProgressiveImageLoaderModule;
}());

/*
 * Public API Surface of ngx-progressive-image-loader
 */

/**
 * Generated bundle index. Do not edit.
 */

export { ConfigurationServiceFactory, DEFAULT_IMAGE_LOADER_OPTIONS, IMAGE_LOADER_CONFIG_TOKEN, ImagePlaceholderComponent, NgxProgressiveImageLoaderModule, ProgressiveImageDirective, ProgressiveImageLoaderComponent, isImagePicture, isPictureElement, isSpider, isSupportIntersectionObserver, loadImage, setAttribute, ConfigurationService as ɵa };
//# sourceMappingURL=ngx-progressive-image-loader.js.map
